travelmathApp.controller('locationsCtrl', ['$scope', '$http', '$location', 'carrier', 'travelMathRequest', 'uiGmapGoogleMapApi', '$ionicScrollDelegate', '$timeout', '$ionicHistory', '$q', function($scope, $http, $location, carrier, travelMathRequest, uiGmapGoogleMapApi, $ionicScrollDelegate, $timeout, $ionicHistory, $q) {
    $scope.data = carrier.getData();
    $scope.buttons = true;
    $scope.flightMapRendered = false;
    $scope.driveMapRendered = false;
    $scope.flightMapVisible = false;
    $scope.driveMapVisible = false;
    $scope.isIos = ionic.Platform.isIOS();
    $scope.isAndroid = ionic.Platform.isAndroid();
    $scope.updateGeodecMap = true;
    $scope.updateDrivingMap = true;
    $scope.beforeInputChange = {ToLocation:'',FromLocation:''};
    $scope.view = {};

    // If the data is empty we have no business on this page
    if ( !$scope.data ) {
        $location.path('/');
        return;
    }

    // save reference to original inputs before change
    $scope.beforeInputChange = {ToLocation:$scope.data.ToLocation,FromLocation:$scope.data.FromLocation};

    // Update locations scope to show hide flight or driving tabs
    updateTabs($scope.data);

    $scope.$watch('data', function() {
        $scope.updateGeodecMap = true;
        $scope.updateDrivingMap = true;
    });

    function updateTabs(data) {
        $scope.view.buttons = true;

        if( data.DrivableRoute === 'No' ) {
            $scope.view.drivable = false;
        } else {
            $scope.view.drivable =  true;
        }

        if( data.ModeTransport === 'Driving' ) {
            $scope.view.flight = false;
            $scope.view.drive = true;
        } else {
            $scope.view.flight = true;
            $scope.view.drive = false;
        }

        if( data.HideFlights === 'Yes' ) {
            $scope.view.flights = false;
            $scope.view.flight = false;
            $scope.view.drive = true;
        } else {
            $scope.view.flights = true;
        }
    }

    $scope.openMaps = function() {
        if( $scope.isIos ) {
            cordova.InAppBrowser.open('http://maps.apple.com/?daddr='+ $scope.data.ToLocation +'&saddr='+ $scope.data.FromLocation+'', '_system', 'location=yes');
        }

        if( $scope.isAndroid ) {
            cordova.InAppBrowser.open('http://maps.google.com/?daddr='+ $scope.data.ToLocation +'&saddr='+ $scope.data.FromLocation+'', '_system', 'location=yes');
        }
    }

    $scope.toggleFlight = function() {
        $scope.view.flight = true;
        $scope.view.drive = false;
        if( $scope.flightMapRendered && $scope.flightMapVisible) {
            $timeout(function() {
                $scope.drawGeodecicMaps();
            },100);
        }
    };

    $scope.toggleDrive = function() {
        $scope.view.flight = false;
        $scope.view.drive = true;
        if( $scope.driveMapRendered && $scope.driveMapVisible) {
            $timeout(function() {
                $scope.drawDrivingMaps();
            },100);
        }
    };

    $scope.toggleDestination = function() {
        var FromLocation = $scope.data.FromLocation,
            ToLocation = $scope.data.ToLocation;

        $scope.data.FromLocation = ToLocation;
        $scope.data.ToLocation = FromLocation;
    };

    $scope.backButton = function() {
        $ionicHistory.goBack();
        // $location.path('/');
    };

    $scope.calculate = function(values, updateTabsOrNot) {

        var data = values || $scope.data;

        if( $scope.SameLocations(data) ) {
            return;
        }
        // show loading
        $scope.locationLoading = true;
        $scope.viewClone = angular.copy($scope.view);
        $scope.view.buttons = false;
        $scope.view.flight = false;
        $scope.view.drive = false;

        travelMathRequest.calculate(data, false).then(function(response) {

            $scope.data = response;
            var oppTest = $scope.oppositeLocations(data);

            if( updateTabsOrNot !== false && !oppTest ) {
                // Update locations scope to show hide flight or driving tabs
                updateTabs($scope.data);
            } else {
                $scope.view = angular.copy($scope.viewClone);
            }


            if( !oppTest ) {
                // hide the maps on new to and from, ignoring if the locations are just opposites from before
                $scope.flightMapVisible = false;
                $scope.driveMapVisible = false;
                // $scope.resetMaps();
            }

            $timeout(function() {
                if( $scope.view.flight && $scope.flightMapRendered && $scope.flightMapVisible ) {
                    $scope.drawGeodecicMaps();
                }

                if( $scope.view.drive && $scope.driveMapRendered && $scope.driveMapVisible  ) {
                    $scope.drawDrivingMaps();
                }
            },100);

            // save reference to original inputs before toggle
            $scope.beforeInputChange = {ToLocation:$scope.data.ToLocation,FromLocation:$scope.data.FromLocation};
            $scope.locationLoading = false;
        }, function() {
            //Ther was an error, nothing to do carrier will handle it
            $scope.locationLoading = false;
            //clear the befores to prevent issue when there is an error and the user type the same locations as before
            $scope.beforeInputChange = angular.copy({ToLocation:'',FromLocation:''});
        });
    };

    $scope.SameLocations = function(data) {
        if ($scope.beforeInputChange.ToLocation === data.ToLocation && $scope.beforeInputChange.FromLocation === data.FromLocation) {return true;}
        return false;
    };
    $scope.oppositeLocations = function(data) {
        if ($scope.beforeInputChange.ToLocation === data.FromLocation && $scope.beforeInputChange.FromLocation === data.ToLocation ) { return true; }
        return false;
    };

    // Listen to setting change mpg and refresh data
    $scope.$on('refresh-required', function(event, args) {
        // clear so we can update the mpg and gas cost
        // $scope.beforeInputChange = {ToLocation:'',FromLocation:''};
        // $scope.calculate($scope.data, false);
        var obj = travelMathRequest.getCostAndUsage($scope.data);
        $scope.data.CostOfDriving = obj.CostOfDriving;
        $scope.data.FuelUsage = obj.FuelUsage;
    });

    uiGmapGoogleMapApi.then(function(maps) {
        var directionsService = new maps.DirectionsService(),
            directionsRenderer = new maps.DirectionsRenderer();

        //provide options, not all work
        $scope.mapOptions = {
            // minZoom: 13,
            // maxZoom: 20,
            zoom: 4,
            bounds: {},
            center: new maps.LatLng($scope.data.StartLatitude, $scope.data.StartLongitude),
            mapTypeId: maps.MapTypeId.ROADMAP,
            streetViewControl: false,
            mapTypeControl: false,
            optimized: true,
            panControl: false,
            zoomControl: true,
            draggable: true,
        };

        $scope.rederFlightMap = function() {
            var mapCanvasDesic = document.getElementById('map-canvas-desic');

            mapCanvasDesic.style.height = '250px';

            // create a map and assign it to the element
            map = new maps.Map(mapCanvasDesic, $scope.mapOptions);

            // map.controls[maps.ControlPosition.TOP_CENTER].push(document.getElementById('info')); // Add controls
            // create marker 1
            marker1 = new maps.Marker({
                map: map,
                draggable: false,
                position: new maps.LatLng($scope.data.StartLatitude, $scope.data.StartLongitude),
                optimized: false,
                icon: {url:'http://maps.google.com/mapfiles/dd-start.png'}
            });
            // create marker 2
            marker2 = new maps.Marker({
                map: map,
                draggable: false,
                position: new maps.LatLng($scope.data.EndLatitude, $scope.data.EndLongitude),
                optimized: false,
                icon: {url:'http://maps.google.com/mapfiles/dd-end.png'}
            });

            // Polyline options
            var geodesicOptions = {
                strokeColor: '#1D60B4',
                strokeOpacity: 1.0,
                strokeWeight: 3,
                geodesic: true,
                map: map
            };
            geodesicPoly = new maps.Polyline(geodesicOptions);
            $scope.flightMapRendered = true;

            $ionicScrollDelegate.resize();
        }

        $scope.rederDriveMap = function() {

            // Driving Directions
            // Directions Functions https://developers.google.com/maps/documentation/javascript/examples/directions-panel
            var mapCanvasDir = document.getElementById('map-canvas-dir');

            mapCanvasDir.style.height = '250px';

            drmap = new maps.Map(mapCanvasDir, $scope.mapOptions);
            directionsRenderer.setMap(drmap);

            directionsRenderer.setPanel(document.getElementById('directions-panel'));

            $scope.driveMapRendered = true;

            $ionicScrollDelegate.resize();
        };

        // Draw maps on scope change
        $scope.drawGeodecicMaps = function(doScroll) {

            if( !$scope.updateGeodecMap ) {
                maps.event.trigger(map, 'resize');
                if( doScroll ) {
                    $ionicScrollDelegate.scrollBottom();
                }
                return;
            }

            if( !$scope.flightMapRendered ) {
                $scope.rederFlightMap();
                $scope.flightMapRendered = true;
            }

            $scope.flightMapVisible = true;
            // marker1.setMap(map);
            // marker2.setMap(map);
            // geodesicPoly.setMap(map);

            marker1.setPosition( new maps.LatLng($scope.data.StartLatitude, $scope.data.StartLongitude) );
            marker2.setPosition( new maps.LatLng($scope.data.EndLatitude, $scope.data.EndLongitude) );

            var path = [marker1.getPosition(), marker2.getPosition()];
            geodesicPoly.setPath(path);

            $timeout(function() {
                maps.event.trigger(map, 'resize');
                // create the bounds rectangle and fit
                var bounds = new maps.LatLngBounds();
                bounds.extend(marker1.getPosition());
                bounds.extend(marker2.getPosition());
                map.fitBounds(bounds);
                // map.setCenter(bounds.getCenter());
                // map.panToBounds(bounds);
            },100);

            $scope.updateGeodecMap = false;
            if( doScroll ) {
                $timeout(function() {
                    $ionicScrollDelegate.scrollBottom();
                },100);
            }
        };


        $scope.drawDrivingMaps = function(doScroll) {
            var rect = document.getElementById('map-canvas-dir').getBoundingClientRect();

            if( !$scope.updateDrivingMap ) {
                maps.event.trigger(drmap, 'resize');
                if( doScroll ) {
                    $ionicScrollDelegate.scrollTo(0,rect.top,true);
                }
                return;
            }

           if( !$scope.driveMapRendered ) {
                $scope.rederDriveMap();
                $scope.driveMapRendered = true;
            }

            $scope.driveMapVisible = true;
            // directionsRenderer.setMap( drmap );
            // directionsRenderer.setOptions( { suppressMarkers: false } );
            // directionsRenderer.setOptions( { suppressPolylines: false } );

            var request = {
                origin: $scope.data.StartLatitude + ',' + $scope.data.StartLongitude,
                destination: $scope.data.EndLatitude + ',' + $scope.data.EndLongitude,
                travelMode: maps.TravelMode.DRIVING
            };

            directionsService.route(request, function(response, status) {
                if( status === maps.DirectionsStatus.OK) {
                    maps.event.trigger(drmap, 'resize');
                    directionsRenderer.setDirections(response);
                    var bounds = response.routes[0].bounds;
                    drmap.fitBounds(bounds);
                    drmap.setCenter(bounds.getCenter());
                }
            });

            $scope.updateDrivingMap = false;
            if( doScroll ) {
                $timeout(function() {
                    var rect = document.getElementById('map-canvas-dir').getBoundingClientRect();
                    $ionicScrollDelegate.scrollTo(0,rect.top,true);
                },100);
            }
        };

        $scope.resetMaps = function() {

            if( $scope.flightMapRendered ) {
                map.setZoom(4);
                map.setCenter(new google.maps.LatLng(40.2093958, -99.6079172));
                marker1.setMap(null);
                marker2.setMap(null);
                geodesicPoly.setMap(null);
            }

            if( $scope.driveMapRendered ) {
                directionsRenderer.setMap( drmap );
                directionsRenderer.setOptions( { suppressMarkers: true } );
                directionsRenderer.setOptions( { suppressPolylines: true } );
                drmap.setZoom(4);
                drmap.setCenter(new google.maps.LatLng(40.2093958, -99.6079172));
            }
        }

    });// end google promise

}]);